@@ -2,19 +2,16 @@
 from __future__ import annotations
 import transformer_lens.utils as utils
 from transformer_lens.utils import Slice, SliceInput
+from jaxtyping import Float
 import torch
 import einops
 from fancy_einsum import einsum
 from typing import Optional, Union, Dict
 from typing_extensions import Literal
-from torchtyping import TensorType as TT
 import re
 import numpy as np
 import logging
 
-from transformer_lens.torchtyping_helper import T
-
-
 class ActivationCache:
     """
     A wrapper around a dictionary of cached activations from a model run, with a variety of helper functions. In general, any utility which is specifically about editing/processing activations should be a method here, while any utility which is more general should be a function in utils.py, and any utility which is specifically about model weights should be in HookedTransformer.py or components.py.
@@ -80,6 +77,9 @@ class ActivationCache:
                     key = (key[0], self.model.cfg.n_layers + key[1], *key[2:])
             return self.cache_dict[utils.get_act_name(*key)]
 
+    def __len__(self):
+        return len(self.cache_dict)
+
     def to(self, device: Union[str, torch.device], move_model=False) -> ActivationCache:
         """
         Moves the cache to a device - mostly useful for moving it to CPU after model computation finishes to save GPU memory. Matmuls will be much slower on the CPU.
@@ -143,7 +143,7 @@ class ActivationCache:
         pos_slice: Union[Slice, SliceInput] = None,
         mlp_input: bool = False,
         return_labels: bool = False,
-    ) -> TT[T.layers_covered, T.batch_and_pos_dims:..., T.d_model]:
+    ) -> Float[torch.Tensor, "layers_covered *batch_and_pos_dims d_model"]:
         """Returns the accumulated residual stream up to a given layer, ie a stack of previous residual streams up to that layer's input. This can be thought of as a series of partial values of the residual stream, where the model gradually accumulates what it wants.
 
         Args:
@@ -182,6 +182,65 @@ class ActivationCache:
         else:
             return components
 
+    def logit_attrs(
+            self,
+            residual_stack: TT[T.num_components, T.batch_and_pos_dims:..., T.d_model],
+            tokens: Union[str, int, TT[()], TT[T.batch], TT[T.batch, T.position]],
+            incorrect_tokens: Union[str, int, TT[()], TT[T.batch], TT[T.batch, T.position]] = None,
+            pos_slice: Union[Slice, SliceInput] = None,
+            batch_slice: Union[Slice, SliceInput] = None,
+            has_batch_dim: bool = True
+        ) -> TT[T.num_components, T.batch_and_pos_dims:...]:
+        """Returns the logit attributions for the residual stack on an input of tokens, or the logit difference attributions for the residual stack if incorrect_tokens is provided.
+
+        Args:
+            residual_stack (TT["num_components", "batch_and_pos_dims":..., "d_model"]): stack of components of residual stream to get logit attributions for.
+            tokens (Union[str, int, TT[()], TT["batch"], TT["batch", "position"]]): tokens to compute logit attributions on.
+            incorrect_tokens (Union[str, int, TT[()], TT["batch"], TT["batch", "position"]], optional): if provided, compute attributions on logit difference between tokens and incorrect_tokens.
+                Must have the same shape as tokens.
+            pos_slice (Slice, optional): The slice to apply layer norm scaling on.
+                Defaults to None, do nothing.
+            batch_slice (Slice, optional): The slice to take on the batch dimension during layer norm scaling.
+                Defaults to None, do nothing.
+            has_batch_dim (bool, optional): Whether residual_stack has a batch dimension.
+                Defaults to True.
+        Returns:
+            Components: A [num_components, batch_and_pos_dims:...] tensor of the logit attributions or logit difference attributions if incorrect_tokens was provided.
+        """
+        if not isinstance(pos_slice, Slice):
+            pos_slice = Slice(pos_slice)
+
+        if not isinstance(batch_slice, Slice):
+            batch_slice = Slice(batch_slice)
+
+        if isinstance(tokens, str):
+            tokens = torch.as_tensor(self.model.to_single_token(tokens))
+
+        elif isinstance(tokens, int):
+            tokens = torch.as_tensor(tokens)
+
+        logit_directions = self.model.tokens_to_residual_directions(tokens)
+
+        if incorrect_tokens is not None:
+            if isinstance(incorrect_tokens, str):
+                incorrect_tokens = torch.as_tensor(self.model.to_single_token(incorrect_tokens))
+
+            elif isinstance(incorrect_tokens, int):
+                incorrect_tokens = torch.as_tensor(incorrect_tokens)
+
+            if tokens.shape != incorrect_tokens.shape:
+                raise ValueError(f"tokens and incorrect_tokens must have the same shape! (tokens.shape={tokens.shape}, incorrect_tokens.shape={incorrect_tokens.shape})")
+        
+            # If incorrect_tokens was provided, take the logit difference
+            logit_directions = logit_directions - self.model.tokens_to_residual_directions(incorrect_tokens)
+
+
+        scaled_residual_stack = self.apply_ln_to_stack(residual_stack, layer=-1, pos_slice=pos_slice, batch_slice=batch_slice, has_batch_dim=has_batch_dim)
+
+        logit_attrs = einsum("... d_model, ... d_model -> ...", scaled_residual_stack, logit_directions)
+
+        return logit_attrs
+        
     def decompose_resid(
         self,
         layer: Optional[int] = None,
@@ -190,7 +249,7 @@ class ActivationCache:
         pos_slice: Union[Slice, SliceInput] = None,
         incl_embeds: bool = True,
         return_labels: bool = False,
-    ) -> TT[T.layers_covered, T.batch_and_pos_dims:..., T.d_model]:
+    ) -> Float[torch.Tensor, "layers_covered *batch_and_pos_dims d_model"]:
         """Decomposes the residual stream input to layer L into a stack of the output of previous layers. The sum of these is the input to layer L (plus embedding and pos embedding). This is useful for attributing model behaviour to different components of the residual stream
 
         Args:
@@ -271,7 +330,7 @@ class ActivationCache:
         return_labels: bool = False,
         incl_remainder: bool = False,
         pos_slice: Union[Slice, SliceInput] = None,
-    ) -> TT[T.num_components, T.batch_and_pos_dims:..., T.d_model]:
+    ) -> Float[torch.Tensor, "num_components *batch_and_pos_dims d_model"]:
         """Returns a stack of all head results (ie residual stream contribution) up to layer L. A good way to decompose the outputs of attention layers into attribution by specific heads. Note that the num_components axis has length layer x n_heads ((layer head_index) in einops notation)
 
         Assumes that the model has been run with use_attn_results=True
@@ -334,7 +393,7 @@ class ActivationCache:
         activation_name: str,
         layer: int = -1,
         sublayer_type: Optional[str] = None,
-    ) -> TT[T.layers_covered, ...]:
+    ) -> Float[torch.Tensor, "layers_covered ..."]:
         """Returns a stack of all head results (ie residual stream contribution) up to layer L. A good way to decompose the outputs of attention layers into attribution by specific heads. The output shape is exactly the same shape as the activations, just with a leading layers dimension.
 
         Args:
@@ -359,7 +418,7 @@ class ActivationCache:
         layer: int,
         neuron_slice: Union[Slice, SliceInput] = None,
         pos_slice: Union[Slice, SliceInput] = None,
-    ) -> TT[T.batch_and_pos_dims:..., T.num_neurons, T.d_model]:
+    ) -> Float[torch.Tensor, "*batch_and_pos_dims num_neurons d_model"]:
         """Returns the results of for neurons in a specific layer (ie, how much each neuron contributes to the residual stream). Does it for the subset of neurons specified by neuron_slice, defaults to all of them. Does *not* cache these because it's expensive in space and cheap to compute.
 
         Args:
@@ -394,7 +453,7 @@ class ActivationCache:
         neuron_slice: Union[Slice, SliceInput] = None,
         return_labels: bool = False,
         incl_remainder: bool = False,
-    ) -> TT[T.num_components, T.batch_and_pos_dims:..., T.d_model]:
+    ) -> Float[torch.Tensor, "num_components *batch_and_pos_dims d_model"]:
         """Returns a stack of all neuron results (ie residual stream contribution) up to layer L - ie the amount each individual neuron contributes to the residual stream. Also returns a list of labels of the form "L0N0" for the neurons. A good way to decompose the outputs of MLP layers into attribution by specific neurons.
 
         Note that doing this for all neurons is SUPER expensive on GPU memory and only works for small models or short inputs.
@@ -457,11 +516,13 @@ class ActivationCache:
 
     def apply_ln_to_stack(
         self,
-        residual_stack: TT[T.num_components, T.batch_and_pos_dims:..., T.d_model],
+        residual_stack: Float[torch.Tensor, "num_components *batch_and_pos_dims d_model"],
         layer: Optional[int] = None,
         mlp_input: bool = False,
         pos_slice: Union[Slice, SliceInput] = None,
-    ) -> TT[T.num_components, T.batch_and_pos_dims:..., T.d_model]:
+        batch_slice: Union[Slice, SliceInput] = None,
+        has_batch_dim: bool = True
+    ) -> Float[torch.Tensor, "num_components *batch_and_pos_dims d_model"]:
         """Takes a stack of components of the residual stream (eg outputs of decompose_resid or accumulated_resid), treats them as the input to a specific layer, and applies the layer norm scaling of that layer to them, using the cached scale factors - simulating what that component of the residual stream contributes to that layer's input.
 
         The layernorm scale is global across the entire residual stream for each layer, batch element and position, which is why we need to use the cached scale factors rather than just applying a new LayerNorm.
@@ -479,21 +540,33 @@ class ActivationCache:
             mlp_input (bool, optional): Whether the input is to the MLP or attn
                 (ie ln2 vs ln1). Defaults to False, ie ln1. If layer==n_layers,
                 must be False, and we use ln_final
-            pos_slice: The slice to take of positions, if residual_stack is not
+            pos_slice (Slice, optional): The slice to take of positions, if residual_stack is not
                 over the full context, None means do nothing. It is assumed that
                 pos_slice has already been applied to residual_stack, and this
                 is only applied to the scale. See utils.Slice for details.
-                Defaults to None.
+                Defaults to None, do nothing.
+            batch_slice (Slice, optional): The slice to take on the batch dimension.
+                Defaults to None, do nothing.
+            has_batch_dim (bool, optional): Whether residual_stack has a batch dimension.
+                Defaults to True.
         """
         if self.model.cfg.normalization_type not in ["LN", "LNPre"]:
             # The model does not use LayerNorm, so we don't need to do anything.
             return residual_stack
         if not isinstance(pos_slice, Slice):
             pos_slice = Slice(pos_slice)
+        if not isinstance(batch_slice, Slice):
+            batch_slice = Slice(batch_slice)
+
         if layer is None or layer == -1:
             # Default to the residual stream immediately pre unembed
             layer = self.model.cfg.n_layers
-        # First, center the stack
+
+        if has_batch_dim:
+            # Apply batch slice to the stack
+            residual_stack = batch_slice.apply(residual_stack, dim=1)
+
+        # Center the stack
         residual_stack = residual_stack - residual_stack.mean(dim=-1, keepdim=True)
 
         if layer == self.model.cfg.n_layers or layer is None:
@@ -505,6 +578,10 @@ class ActivationCache:
         # The shape of scale is [batch, position, 1] or [position, 1] - final dimension is a dummy thing to get broadcoasting to work nicely.
         scale = pos_slice.apply(scale, dim=-2)
 
+        if self.has_batch_dim:
+            # Apply batch slice to the scale
+            scale = batch_slice.apply(scale)
+
         return residual_stack / scale
 
     def get_full_resid_decomposition(
@@ -515,7 +592,7 @@ class ActivationCache:
         apply_ln: bool = False,
         pos_slice: Union[Slice, SliceInput] = None,
         return_labels: bool = False,
-    ) -> TT[T.num_components, T.batch_and_pos_dims:..., T.d_model]:
+    ) -> Float[torch.Tensor, "num_components *batch_and_pos_dims d_model"]:
         """Returns the full decomposition of the residual stream into embed, pos_embed, each head result, each neuron result, and the accumulated biases. We break down the residual stream that is input into some layer.
 
         Args:
