@@ -1,17 +1,16 @@
 from __future__ import annotations
 import torch
 from typing import Optional, Union, Tuple, List, Dict
-from torchtyping import TensorType as TT
+from jaxtyping import Float
 from functools import lru_cache
 import transformer_lens.utils as utils
-from transformer_lens.torchtyping_helper import T as TH # Need the import as since FactoredMatrix declares T (for transpose)
 
 class FactoredMatrix:
     """
     Class to represent low rank factored matrices, where the matrix is represented as a product of two matrices. Has utilities for efficient calculation of eigenvalues, norm and SVD.
     """
 
-    def __init__(self, A: TT[..., TH.ldim, TH.mdim], B: TT[..., TH.mdim, TH.rdim]):
+    def __init__(self, A: Float[torch.Tensor, "... ldim mdim"], B: Float[torch.Tensor, "... mdim rdim"]):
         self.A = A
         self.B = B
         assert self.A.size(-1) == self.B.size(
@@ -29,8 +28,8 @@ class FactoredMatrix:
         self.B = self.B.broadcast_to(self.shape[:-2] + (self.mdim, self.rdim))
 
     def __matmul__(
-        self, other: Union[TT[..., TH.rdim, TH.new_rdim], TT[TH.rdim], FactoredMatrix]
-    ) -> Union[FactoredMatrix, TT[..., TH.ldim]]:
+        self, other: Union[Float[torch.Tensor, "... rdim new_rdim"], Float[torch.Tensor, "rdim"], FactoredMatrix]
+    ) -> Union[FactoredMatrix, Float[torch.Tensor, "... ldim"]]:
         if isinstance(other, torch.Tensor):
             if other.ndim < 2:
                 # It's a vector, so we collapse the factorisation and just return a vector
@@ -48,8 +47,8 @@ class FactoredMatrix:
             return (self @ other.A) @ other.B
 
     def __rmatmul__(
-        self, other: Union[TT[..., TH.new_rdim, TH.ldim], TT[TH.ldim], FactoredMatrix]
-    ) -> Union[FactoredMatrix, TT[..., TH.rdim]]:
+        self, other: Union[Float[torch.Tensor, "... new_rdim ldim"], Float[torch.Tensor, "ldim"], FactoredMatrix]
+    ) -> Union[FactoredMatrix, Float[torch.Tensor, "... rdim"]]:
         if isinstance(other, torch.Tensor):
             assert (
                 other.size(-1) == self.ldim
@@ -65,12 +64,12 @@ class FactoredMatrix:
             return other.A @ (other.B @ self)
 
     @property
-    def AB(self) -> TT[TH.leading_dims:..., TH.ldim, TH.rdim]:
+    def AB(self) -> Float[torch.Tensor, "*leading_dims ldim rdim"]:
         """The product matrix - expensive to compute, and can consume a lot of GPU memory"""
         return self.A @ self.B
 
     @property
-    def BA(self) -> TT[TH.leading_dims:..., TH.rdim, TH.ldim]:
+    def BA(self) -> Float[torch.Tensor, "*leading_dims rdim ldim"]:
         """The reverse product. Only makes sense when ldim==rdim"""
         assert (
             self.rdim == self.ldim
@@ -85,9 +84,9 @@ class FactoredMatrix:
     def svd(
         self,
     ) -> Tuple[
-        TT[TH.leading_dims:..., TH.ldim, TH.mdim],
-        TT[TH.leading_dims:..., TH.mdim],
-        TT[TH.leading_dims:..., TH.rdim, TH.mdim],
+        Float[torch.Tensor, "*leading_dims ldim mdim"],
+        Float[torch.Tensor, "*leading_dims mdim"],
+        Float[torch.Tensor, "*leading_dims rdim mdim"],
     ]:
         """
         Efficient algorithm for finding Singular Value Decomposition, a tuple (U, S, Vh) for matrix M st S is a vector and U, Vh are orthogonal matrices, and U @ S.diag() @ Vh.T == M
@@ -104,19 +103,19 @@ class FactoredMatrix:
         return U, S, Vh
 
     @property
-    def U(self) -> TT[TH.leading_dims:..., TH.ldim, TH.mdim]:
+    def U(self) -> Float[torch.Tensor, "*leading_dims ldim mdim"]:
         return self.svd()[0]
 
     @property
-    def S(self) -> TT[TH.leading_dims:..., TH.mdim]:
+    def S(self) -> Float[torch.Tensor, "*leading_dims mdim"]:
         return self.svd()[1]
 
     @property
-    def Vh(self) -> TT[TH.leading_dims:..., TH.rdim, TH.mdim]:
+    def Vh(self) -> Float[torch.Tensor, "*leading_dims rdim mdim"]:
         return self.svd()[2]
 
     @property
-    def eigenvalues(self) -> TT[TH.leading_dims:..., TH.mdim]:
+    def eigenvalues(self) -> Float[torch.Tensor, "*leading_dims mdim"]:
         """Eigenvalues of AB are the same as for BA (apart from trailing zeros), because if BAv=kv ABAv = A(BAv)=kAv, so Av is an eigenvector of AB with eigenvalue k."""
         return torch.linalg.eig(self.BA).eigenvalues
 
@@ -138,7 +137,7 @@ class FactoredMatrix:
                 f"{idx} is too long an index for a FactoredMatrix with shape {self.shape}"
             )
 
-    def norm(self) -> TT[TH.leading_dims:...]:
+    def norm(self) -> Float[torch.Tensor, "*leading_dims"]:
         """
         Frobenius norm is sqrt(sum of squared singular values)
         """
@@ -163,13 +162,13 @@ class FactoredMatrix:
     def ndim(self) -> int:
         return len(self.shape)
 
-    def collapse_l(self) -> TT[TH.leading_dims:..., TH.mdim, TH.rdim]:
+    def collapse_l(self) -> Float[torch.Tensor, "*leading_dims mdim rdim"]:
         """
         Collapses the left side of the factorization by removing the orthogonal factor (given by self.U). Returns a (..., mdim, rdim) tensor
         """
         return self.S[..., :, None] * utils.transpose(self.Vh)
 
-    def collapse_r(self) -> TT[TH.leading_dims:..., TH.ldim, TH.mdim]:
+    def collapse_r(self) -> Float[torch.Tensor, "*leading_dims ldim mdim"]:
         """
         Analogous to collapse_l, returns a (..., ldim, mdim) tensor
         """
@@ -182,6 +181,6 @@ class FactoredMatrix:
     def pair(
         self,
     ) -> Tuple[
-        TT[TH.leading_dims:..., TH.ldim, TH.mdim], TT[TH.leading_dims:..., TH.mdim, TH.rdim]
+        Float[torch.Tensor, "*leading_dims ldim mdim"], Float[torch.Tensor, "*leading_dims mdim rdim"]
     ]:
         return (self.A, self.B)
\ No newline at end of file
