@@ -3,14 +3,13 @@ import torch.nn as nn
 from dataclasses import dataclass
 from typing import Union, Tuple, List, Dict, Any, Optional
 from transformer_lens.HookedTransformerConfig import HookedTransformerConfig
-from torchtyping import TensorType as TT
-from transformer_lens.torchtyping_helper import T
+from jaxtyping import Float 
 
 
 @dataclass
 class HookedTransformerKeyValueCacheEntry:
-    past_keys: TT[T.batch, T.pos_so_far, T.n_heads, T.d_head]
-    past_values: TT[T.batch, T.pos_so_far, T.n_heads, T.d_head]
+    past_keys: Float[torch.Tensor, "batch pos_so_far n_heads d_head"]
+    past_values: Float[torch.Tensor, "batch pos_so_far n_heads d_head"]
 
     @classmethod
     def init_cache_entry(
@@ -30,14 +29,14 @@ class HookedTransformerKeyValueCacheEntry:
 
     def append(
         self,
-        new_keys: TT[T.batch, T.new_tokens, T.n_heads, T.d_head],
-        new_values: TT[T.batch, T.new_tokens, T.n_heads, T.d_head],
+        new_keys: Float[torch.Tensor, "batch new_tokens n_heads d_head"],
+        new_values: Float[torch.Tensor, "batch new_tokens n_heads d_head"],
     ):
-        updated_keys: TT[
-            "batch", "pos_so_far + new_tokens", "n_heads", "d_head"
+        updated_keys: Float[torch.Tensor, 
+            "batch pos_so_far_plus_new_tokens n_heads d_head"
         ] = torch.cat([self.past_keys, new_keys], dim=1)
-        updated_values: TT[
-            "batch", "pos_so_far + new_tokens", "n_heads", "d_head"
+        updated_values: Float[torch.Tensor,
+            "batch pos_so_far_plus_new_tokens n_heads d_head"
         ] = torch.cat([self.past_values, new_values], dim=1)
         self.past_keys = updated_keys
         self.past_values = updated_values
